@@ -2,7 +2,7 @@
 #
 # Graphical interface for the eSpeak speech synthesizer
 #
-# Copyright © 2009-2011 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
+# Copyright © 2009-2012 Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>
 # Copyright © 2009 Joe Burmeister <joe.a.burmeister@googlemail.com>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -25,62 +25,29 @@ import gobject
 import gtksourceview2
 import gettext
 import locale
+import gtkspell
 from optparse import OptionParser
 from espeak import espeak
 
 from resources import *
+import about_dialog
+from language import LanguageManager
+from voicebox import VoiceBox
 
 gtk.gdk.threads_init()
-gettext.install('espeak-gui', unicode=True)
-locale.setlocale(locale.LC_ALL, "") # for unicode sorting with locale.strcoll
-
-_gnulicense = \
-"This program is free software: you can redistribute it and/or modify\n\
-it under the terms of the GNU General Public License as published by\n\
-the Free Software Foundation, either version 3 of the License, or\n\
-(at your option) any later version.\n\
-\n\
-This program is distributed in the hope that it will be useful,\n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
-GNU General Public License for more details.\n\
-\n\
-You should have received a copy of the GNU General Public License\n\
-along with this program.  If not, see <http://www.gnu.org/licenses/>."
 
 class Application:
 
-    SEPARATOR_KEY = 'separator'
-
     _espeak_language = None
+    _language_manager = None
+    _voicebox = None
+    _spell = None
     
     @property
     def _espeak_gender(self):
         gender = settings.get_string(settings_gender)
         return espeak.Gender.Male if gender == 'male' else (
-        	espeak.Gender.Female if gender == 'female' else None)
-
-    def _create_about_dialog(self):
-        dialog = gtk.AboutDialog()
-        dialog.set_version(version)
-        dialog.set_program_name('espeak-gui')
-        dialog.set_icon(self._main.get_icon())
-        dialog.set_license(_gnulicense)
-        dialog.set_copyright(
-            'Copyright © 2009-2011 Siegfried-Angel Gevatter Pujals\n' \
-            'Copyright © 2009 Joe Burmeister')
-        dialog.set_authors([
-            'Siegfried-Angel Gevatter Pujals <siegfried@gevatter.com>',
-            'Joe Burmeister <joe.a.burmeister@googlemail.com>'])
-        dialog.set_logo(dialog.get_icon())
-        
-        def close(w, res=None):
-            w.hide()
-        
-        dialog.connect('response', close)
-        dialog.connect('destroy', close)
-        
-        return dialog
+            espeak.Gender.Female if gender == 'female' else None)
     
     def __init__(self):
         self._builder = gtk.Builder()
@@ -109,11 +76,10 @@ class Application:
         self._main.set_icon_from_file(icon_file)
         
         self._scrollbox = self._builder.get_object('scrolledwindow')
-        self._voicebox = self._builder.get_object('voicebox')
         self._playbutton = self._builder.get_object('button_read')
         self._pausebutton = self._builder.get_object('button_pause')
         self._speedspin = self._builder.get_object('speedspin')
-        self._about_dialog = self._create_about_dialog()
+        self._about_dialog = about_dialog.create_about_dialog(self._main)
         
         self._buffer = gtksourceview2.Buffer()
         self._buffer.connect('changed', self._e_text_changed)
@@ -125,26 +91,18 @@ class Application:
         self._inputbox.set_wrap_mode(gtk.WRAP_WORD);
         self._scrollbox.add(self._inputbox)
         #self._inputbox.connect('populate-popup', self._e_on_context_menu)
+                
+        self._language_manager = LanguageManager(self._get_voice_list())
+        self._voicebox = VoiceBox(
+            self._builder.get_object('voicebox'), self._language_manager)
+        self._voicebox.set_voice(
+            settings.get_string(settings_selected_voice) or \
+            self._language_manager.get_default_language())
         
         self._playing = False
         self._paused = False
         self._speedspin.set_value(settings.get_int(settings_speed) or 160)
         
-        self._get_language_list()
-        store = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING)
-        for voice in self._languages_other:
-            store.append([voice.name.capitalize(), voice.identifier])
-        self._voicebox.set_model(store)
-        cellr = gtk.CellRendererText()
-        self._voicebox.pack_start(cellr)
-        self._voicebox.add_attribute(cellr, 'text', 0)
-        self._voicebox.set_row_separator_func(lambda model, _iter:
-            model.get_value(_iter, 1) == self.SEPARATOR_KEY)
-        self._set_voice(
-            settings.get_string(settings_selected_voice) or \
-            'English') # Do not translate the "English" string
-        self._voicebox.connect('key-press-event', self._on_voicebox_keypress)
-        
         espeak.set_SynthCallback(self._e_eventCB)
         
         self._speaking_tag = self._buffer.create_tag('speaking')
@@ -161,17 +119,22 @@ class Application:
         self._pausebutton.set_visible(False)
         gtk.main()
     
-    def _get_language_list(self):
-        self._languages_other = []
-        for voice in self._get_voice_list():
-            self._languages_other.append(voice)
+    def _autodetect(self):
+        text = self._buffer.get_text(self._buffer.get_start_iter(),
+            self._buffer.get_end_iter())
+        return self._language_manager.autodetect(text)
     
     def synth(self, text):
         parameters = {}
-        for param in ('language', 'gender'):
-            value = getattr(self, '_espeak_%s' % param)
-            if value:
-                parameters[param] = value
+        if self._espeak_gender:
+            parameters['gender'] = self._espeak_gender
+        if self._espeak_language not in (None, 'autodetect'):
+            parameters['language'] = self._espeak_language
+        else:
+            guess = self._autodetect()
+            self._voicebox.set_guess(guess)
+            parameters['language'] = guess.get_language()
+        print parameters
         espeak.set_voice(**parameters)
         return espeak.synth(text)
     
@@ -180,50 +143,22 @@ class Application:
         return sorted(espeak.list_voices(), cmp=locale.strcoll,
             key=lambda voice: voice.name)
     
-    def _find_voice_position(self, start=None, complete=None, from_position=0):
-        """ Returns the position of the first voice whose name matches
-            the value provided in `complete` or starts with the character
-            given in `start`.
-            
-            The optional parameter `from_position' sets the position from
-            which _find_voice_position should start searching. If it is
-            different from zero, _find_voice_position will start searching
-            again from the start, until completing the circle (reaching the
-            value in from_position).
-            
-            Only one of `start` and `complete` can be used at the same time. """
-        
-        store = self._voicebox.get_model()
-        needle = start or complete
-        values = [(x[0][0] if start else x[0]) for x in store]
+    def _e_voicebox_changed(self, widget):
+        (voice_name, voice_id) = self._voicebox.get_voice()
+        self._active_voice = voice_name
+        self._espeak_language = voice_id
+        settings.set_string(settings_selected_voice, voice_id)
+        if not self._spell:
+            self._spell = gtkspell.Spell(self._inputbox)
         try:
-            return values[from_position:].index(needle) + from_position
-        except ValueError:
-        	try:
-        		return values[:from_position].index(needle)
-        	except ValueError:
-	            return -1
+            self._spell.set_language(voice_id.split('/', 1)[-1])
+        except RuntimeError:
+           # unsupported language
+            self._spell.detach()
+            self._spell = None
     
     def _set_voice(self, voice):
-        self._voicebox.set_active(self._find_voice_position(
-            complete=voice.capitalize()))
-    
-    def _on_voicebox_keypress(self, voicebox, event):
-        """ Iterates the selection over the languages starting with the
-            typed character. """
-        position = self._voicebox.get_active()
-        store = self._voicebox.get_model()
-        letter = event.string.upper()
-        if not letter:
-            return
-        self._voicebox.set_active(
-           	 self._find_voice_position(start=letter, from_position=position+1))
-    
-    def _e_voicebox_changed(self, widget):
-        language = widget.get_model()[self._voicebox.get_active()]
-        self._active_voice = language[0]
-        self._espeak_language = language[1]
-        settings.set_string(settings_selected_voice, language[0])
+        self._voicebox.set_voice(voice)
     
     def _cleanup_speech_end(self):
         self._playing = False
@@ -235,6 +170,7 @@ class Application:
         self._playbutton.set_label(_('Play'))
         self._pausebutton.set_label(_('Pause'))
         self._pausebutton.set_visible(False)
+        self._playbutton.show()
     
     def _e_eventCB(self, event, pos, length):
         
@@ -317,6 +253,8 @@ class Application:
         else:
             espeak.cancel()
             self._cleanup_speech_end()
+        
+        self._playbutton.show()
     
     def _e_button_pause_cd(self, *discard):
         if self._paused:
@@ -495,7 +433,7 @@ class Application:
     #            languages.append(item)
     #    # Make all new entries visible
     #    menu.show_all()
-	
+    
     def _e_quit(self, *discard):
         espeak.set_SynthCallback(None)
         gtk.main_quit()
