@@ -21,26 +21,25 @@
 import os
 import sys
 import gconf
+import locale
+import gettext
 import logging
 import optparse
 import subprocess
 
-# Logging
-
-logging.basicConfig(
-    format = '%(message)s',
-    level = logging.DEBUG,
-    )
-
-output = logging.getLogger()
-
 # Constants
 
 version = '0.3'
 basedir = os.path.join(os.path.dirname(__file__), '..')
+# It's really a .ui file, but distutils i18n wants .glade
 interface_file = os.path.join(basedir, 'data/interface.glade')
 icon_file = os.path.join(basedir, 'data/espeak-gui.png')
 
+# Internationalization
+
+gettext.install('espeak-gui', unicode=True)
+locale.setlocale(locale.LC_ALL, "") # for unicode sorting with locale.strcoll
+
 # GConf
 
 settings = gconf.Client()
@@ -62,9 +61,9 @@ def new_instance():
     subprocess.Popen([sys.argv[0]])
 
 __all__ = [
-    'version', 'output',
+    'version',
     'interface_file', 'icon_file',
     'settings', 'settings_selected_voice', 'settings_speed',
     'settings_punctuation', 'settings_gender',
-    'options', 'arguments', 'new_instance',
+    'options', 'arguments', 'new_instance'
 ]
